%   This program simulates data from the posterior distribution of the linear VAR
%   The simulation code was adapted from Anders Warne's 'simulate.prg' Rats code.     
clear all;clc;
load newgdpd.txt;
data = newgdpd;
freq = 4;       % The frequency of the data
begin_yr = 1960;% The first data point to be used (before trimming)
begin_per=1;    
end_yr = 2001;  % The last data point to be used
end_per = 4;

cstruct = cal(begin_yr,begin_per,4); 
begf = ical(1960,2,cstruct);
endf = ical(end_yr,end_per,cstruct);

aus = data(:,2);
usa = data(:,3);
gbr = data(:,4);
ger = data(:,5);
can = data(:,6);
jpn = data(:,7);

yt = [aus usa gbr ger can jpn];


load draws\g0draws_var.mat; results.g0 = data; 
load draws\n0draws_var.mat; results.n0 = data; 
load draws\sigdraws_var.mat; results.sig = data; 
load draws\psidraws_var.mat; results.psi = data; 
load draws\pidraws_var.mat; results.pimat = data; clear data;


yactual = log(yt);
 

% Set various constants
nsim    =  rows(results.g0);% The number of simulations
nseries = cols(yt);         % The number of series
nlags   = 1;                % The number of lags (in VECM parameterisation)
stfreq = 1;                 % Write data to files every stfreq draws
nobs =  rows(yactual);      % The number of observations



ydiff =[ zeros(nlags,nseries); 100*(yactual(2:nobs,:) - yactual(1:nobs-1,:))];
realitycheck1 = min(ydiff);
realitycheck2 = max(ydiff);
realityfactor = 4;
realityvector = [];

simdata_au =  [];
simdata_us =  [];
simdata_uk =  [];
simdata_ge =  [];
simdata_ca =  [];
simdata_jp =  [];

simlevel_au =  [];
simlevel_us =  [];
simlevel_uk =  [];
simlevel_ge =  [];
simlevel_ca =  [];
simlevel_jp =  [];


hwait = waitbar(0,'simulating data ...');
tmpst=1;
for jj=1:nsim
    if tmpst == stfreq 
        accept = 0; 
        %   PSI_MAT terms
        psitt = reshape(results.psi(jj,:),nseries,nseries);      
        %   N0
        n0tt  = results.n0(jj,:);
        g0tt  = results.g0(jj,:);
        pitt  = reshape(results.pimat(jj,:),nseries,nseries);
        %   Vi = covariance matrices (when VCV is not state dependent, V0 = V1)
        sig = zeros(nseries,nseries);
        sig(1,1) = results.sig(jj,1);
        sig(2,1:2) = results.sig(jj,2:3);
        sig(3,1:3) = results.sig(jj,4:6);
        sig(4,1:4) = results.sig(jj,7:10);
        sig(5,1:5) = results.sig(jj,11:15);
        sig(6,:) = results.sig(jj,16:21);
        sig(1,2:end) = sig(2:end,1)';
        sig(2,3:end) = sig(3:end,2)';
        sig(3,4:end) = sig(4:end,3)';
        sig(4,5:end) = sig(5:end,4)';
        sig(5,6) = sig(6,5);
        
        
        %   simulate series
        ylevel = zeros(size(yactual));
        dzsim = zeros(size(ylevel));
        zlevel = zeros(size(ylevel));
        ylevel(1:nlags+1,:) = repmat(n0tt,nlags+1,1) ;
        ysim = ydiff;
        zlevel(1:nlags+1,:) = 100*yactual(1:nlags+1,:)-repmat(n0tt,nlags+1,1);
        dzsim(2:nlags+1,:) = zlevel(2:nlags+1,:)-zlevel(1:nlags,:);
        
        dylag = ydiff(nlags,:);
        % draw residuals
        A = chol(sig);
        innov = randn(nobs,nseries)*A;
        
        ii = nlags+1; 
        
        for tt = ii:nobs;
            % construct dztt...
            dzsim(tt,:) = [zlevel(tt-1,:) dzsim(tt-1,:)]*[pitt; psitt] + innov(tt,:);
            
            % then ztt...
            zlevel(tt,:) = zlevel(tt-1,:) + dzsim(tt,:);
        end
        % and finally ytt
        ylevel = ([ones(nobs,1) seqa(1,1,nobs)]*[n0tt; g0tt]) + zlevel;
        ysim(nlags+1:nobs,:) = (ylevel(nlags+1:nobs,:)-ylevel(nlags:nobs-1,:));
        
        % perform "reality check"
        duh_au = min(ysim(:,1)) < 0 & max(ysim(:,1))>0 & ...
            min(ysim(:,1))>=realityfactor*realitycheck1(1) & max(ysim(:,1))<=realityfactor*realitycheck2(1);
        
        duh_us = min(ysim(:,2)) < 0 & max(ysim(:,2))>0 & ...
            min(ysim(:,2))>=realityfactor*realitycheck1(2) & max(ysim(:,2))<=realityfactor*realitycheck2(2);
        
        duh_uk = min(ysim(:,3)) < 0 & max(ysim(:,3))>0 & ...
            min(ysim(:,3))>=realityfactor*realitycheck1(3) & max(ysim(:,3))<=realityfactor*realitycheck2(3);
        
        duh_ge = min(ysim(:,4)) < 0 & max(ysim(:,4))>0 & ...
            min(ysim(:,4))>=realityfactor*realitycheck1(4) & max(ysim(:,4))<=realityfactor*realitycheck2(4);
        
        duh_ca = min(ysim(:,5)) < 0 & max(ysim(:,5))>0 & ...
            min(ysim(:,5))>=realityfactor*realitycheck1(5) & max(ysim(:,5))<=realityfactor*realitycheck2(5);
        
        duh_jp = min(ysim(:,6)) < 0 & max(ysim(:,6))>0 & ...
            min(ysim(:,6))>=realityfactor*realitycheck1(6) & max(ysim(:,6))<=realityfactor*realitycheck2(6);
        % check for at least one turning point, imposing some realism
        
        
        ylevel = (ylevel/100);
        
        % now put back on scale of original data
        realityvector = [ realityvector ; duh_au duh_us duh_uk duh_ge duh_ca duh_jp];
        
        simdata_au = [ simdata_au; ysim(:,1)'];
        simdata_us = [ simdata_us; ysim(:,2)'];
        simdata_uk = [ simdata_uk; ysim(:,3)'];
        simdata_ge = [ simdata_ge; ysim(:,4)'];
        simdata_ca = [ simdata_ca; ysim(:,5)'];
        simdata_jp = [ simdata_jp; ysim(:,6)'];
        
        simlevel_au = [ simlevel_au; ylevel(:,1)'];
        simlevel_us = [ simlevel_us; ylevel(:,2)'];
        simlevel_uk = [ simlevel_uk; ylevel(:,3)'];
        simlevel_ge = [ simlevel_ge; ylevel(:,4)'];
        simlevel_ca = [ simlevel_ca; ylevel(:,5)'];
        simlevel_jp = [ simlevel_jp; ylevel(:,6)'];
        tmpst =1;
    else
        tmpst   = tmpst +1;
    end 
    
    waitbar(jj/nsim);
end

close(hwait);

% save simulated data series
save simulated_data\simlevel_au_var.txt simlevel_au -ascii;
save simulated_data\simdata_au_var.txt simdata_au -ascii;


save simulated_data\simlevel_us_var.txt simlevel_us -ascii;
save simulated_data\simdata_us_var.txt simdata_us -ascii;

save simulated_data\simlevel_uk_var.txt simlevel_uk -ascii;
save simulated_data\simdata_uk_var.txt simdata_uk -ascii;

save simulated_data\simlevel_ge_var.txt simlevel_ge -ascii;
save simulated_data\simdata_ge_var.txt simdata_ge -ascii;

save simulated_data\simlevel_ca_var.txt simlevel_ca -ascii;
save simulated_data\simdata_ca_var.txt simdata_ca -ascii;

save simulated_data\simlevel_jp_var.txt simlevel_jp -ascii;
save simulated_data\simdata_jp_var.txt simdata_jp -ascii;

save simulated_data\reality_vector_var.txt  realityvector -ascii;

clear all;
