function [Ihat, rhoShat] = synch(S);
%****************************************************************************%
%* Concord1: Concordance Index                                              *%
%* INPUT                                                                    *%
%*    S a (T * p) matrix of states (1=expansion, 0=recession)               *%
%*    p11vec & p00vec are the transition probabilities (means or modes)
%*    from the Markov switching models                                      *%
%*                                                                          *%
%* OUTPUT                                                                   *%
%*    C_t is a (T * 1) vector with typical element 1 if all S(t) are in the *%
%*         same state and zero otherwise                                    *%
%*    Ihat is the concordance index as described in Harding & Pagan's
%*          "syncronization" paper
%*    rhoShat is the correlation coefficient between the two columns of S
%
%****************************************************************************% 
Ct= prod(S')' + prod((1-S)')';
Ihat = mean(Ct); % concordance index

mu=mean(S)';
sig = std(S)';

if any(sig==0)
    rhoShat = -2;
else
    tmp = corrcoef(S);
    rhoShat = tmp(1,2);
end

